<?php

namespace Drupal\fixed_text_link_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\file\Plugin\Field\FieldFormatter\FileFormatterBase;

/**
 * Plugin implementation of the 'file_url_plain' formatter.
 *
 * @FieldFormatter(
 *   id = "fixed_text_file_url",
 *   label = @Translation("Link with a fixed text"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class FixedTextFileUrl extends FileFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'link_text' => 'Download',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $parentForm, FormStateInterface $form_state) {
    $parentForm = parent::settingsForm($parentForm, $form_state);

    $form['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link text'),
      '#default_value' => $this->getLinkText(),
      '#required' => TRUE,
    ];

    return $form + $parentForm;
  }

  /**
   * @return string
   */
  private function getLinkText() {
    return $this->getSettings()['link_text'];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary[] = $this->t('Link text: @text', ['@text' => $this->getTranslatedLinkText()]);

    return $summary;
  }

  /**
   * @return string
   */
  private function getTranslatedLinkText() {
    return $this->t($this->getLinkText(), [], ['context' => 'User entered link title']);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    /** @var \Drupal\file\Entity\File $file */
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {
      $text = $this->getTranslatedLinkText();
      $url = Url::fromUri(file_create_url($file->getFileUri()));
      $link = Link::fromTextAndUrl($text, $url);

      $build = $link->toRenderable();
      $cacheableMetadata = CacheableMetadata::createFromObject($file);
      $cacheableMetadata->applyTo($build);
      $elements[$delta] = $build;
    }

    return $elements;
  }

}
